import Flutter
import UIKit
import EngagementSDK


public class SwiftLivelikeFlutterSdkPlugin: NSObject, FlutterPlugin {
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: "livelike_flutter_sdk", binaryMessenger: registrar.messenger())
        let instance = SwiftLivelikeFlutterSdkPlugin()
        registrar.addMethodCallDelegate(instance, channel: channel)
        let factory = ChatViewFactory(messenger: registrar.messenger(),plugin:instance)
        registrar.register(factory, withId: "chatView")
        let widgetFactory = WidgetViewFactory(messenger: registrar.messenger(),plugin:instance)
        registrar.register(widgetFactory,withId:"widgetView")
        instance.accessTokenDelegate = FlutterEventChannel(name: "flutter_sdk_access_token_delegate",binaryMessenger:registrar.messenger())
        debugPrint("Add event Channel")
        instance.errorDelegate = FlutterEventChannel(name: "flutter_sdk_error_delegate", binaryMessenger: registrar.messenger())
        instance.widgetStreamChannel = FlutterEventChannel(name: "widget_stream", binaryMessenger: registrar.messenger())
        instance.rewardStreamChannel = FlutterEventChannel(name: "reward_stream", binaryMessenger: registrar.messenger())
    }
    
    var engagementSDK: EngagementSDK!
    var sessionList = [String: ContentSession]()
    var chatSessionList = [String: ChatSession]()
    var currentPDTTimeList = [String: Int]()
    var accessTokenDelegate:FlutterEventChannel!
    var errorDelegate:FlutterEventChannel!
    var widgetStreamChannel:FlutterEventChannel!
    var rewardStreamChannel:FlutterEventChannel!
    private let errorHandler = EventStreamHandler()
    let widgetStreamHandler = EventStreamHandler()
    let rewardStreamHandler = EventStreamHandler()
    
    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if (call.method == "getPlatformVersion") {
            result("iOS " + UIDevice.current.systemVersion)
        }else if(call.method=="initSDK"){
            if let args = call.arguments as? NSDictionary,
               let clientId = args["clientId"] as? String{
                let usingAccessToken: Bool = ((args["usingAccessToken"] as? Bool?) ?? false)!
                var config = EngagementSDKConfig(clientID:clientId)
                if ((usingAccessToken)){
                    if let accessToken = args["accessToken"] as? String{
                        config.accessTokenStorage = FlutterUserAccessTokenStorage(eventChannel: accessTokenDelegate,accessToken: accessToken)
                    }
                    else{
                        config.accessTokenStorage = FlutterUserAccessTokenStorage(eventChannel: accessTokenDelegate)
                    }
                }
                rewardStreamChannel.setStreamHandler(rewardStreamHandler)
                engagementSDK = EngagementSDK(config: config)
                errorDelegate.setStreamHandler(errorHandler)
                result(nil)
                return
            }
            result(FlutterError.init(code: "", message: "Client id cannot be empty", details: nil))
        }else if(call.method == "setVideoTimePDT"){
            if let args = call.arguments as? NSDictionary,
               let videoPDTTime = args["videoPDTTime"] as? Int{
                if let programId = args["programId"] as? String {
                    currentPDTTimeList[programId] = videoPDTTime
                }
                if let chatRoomId = args["chatRoomId"] as? String {
                    currentPDTTimeList[chatRoomId] = videoPDTTime
                }
            }
            result(nil)
        }else if(call.method=="createSession"){
            if let args = call.arguments as? NSDictionary,
               let programId = args["programId"] as? String{
                
                let config = SessionConfiguration(programID: programId)
                
                sessionList[programId] = engagementSDK.contentSession(config: config)
                widgetStreamChannel.setStreamHandler(widgetStreamHandler)
                sessionList[programId]!.getChatSession(completion: { [self] (chatResult) in
                    do {
                        let chatSession = try chatResult.get().self
                        let roomId = chatSession.roomID
                        chatSessionList[roomId] = chatSession
                        result(["programId":self.sessionList[programId]?.programID,"chatRoomId": roomId])
                    } catch {
                        errorHandler.eventSink?(error)
                        result(nil)
                    }
                })
                return
            }
            result (nil)
        }
        else if(call.method=="resumeSession"){
            if let args = call.arguments as? NSDictionary,
               let programId = args["programId"] as? String{
                sessionList[programId]?.resume()
                result(nil)
                return
            }
            result(FlutterError.init(code: "", message: "Error in resuming session", details: nil))
        }
        else if(call.method=="pauseSession"){
            if let args = call.arguments as? NSDictionary,
               let programId = args["programId"] as? String{
                sessionList[programId]?.pause()
                result(nil)
                return
            }
            result(FlutterError.init(code: "", message: "Error in pausing session", details: nil))
        }
        else if(call.method=="closeSession"){
            if let args = call.arguments as? NSDictionary,
               let programId = args["programId"] as? String{
                sessionList[programId]?.close()
                sessionList.removeValue(forKey: programId)
                result(nil)
                return
            }
            result(FlutterError.init(code: "", message: "Error in closing session", details: nil))
        }else if(call.method=="createChatSession"){
            if let args = call.arguments as? NSDictionary,
               let chatRoomId = args["chatRoomId"] as? String{
                let config = ChatSessionConfig(roomID: chatRoomId)
                self.engagementSDK.connectChatRoom(config: config) { [weak self] res in
                    guard let self = self else { return }
                    switch res {
                    case .success(let chatSession):
                        self.chatSessionList[chatRoomId] = chatSession
                        result(["chatRoomId":chatSession.roomID])
                    case .failure(let error):
                        self.errorHandler.eventSink?(error)
                    }
                }
                return
            }
            result(FlutterError.init(code: "", message: "Error in creating chat session", details: nil))
        }else if(call.method == "resumeChatSession"){
            if let args = call.arguments as? NSDictionary,
               let chatRoomId = args["chatRoomId"] as? String{
                chatSessionList[chatRoomId]?.resume()
                result(nil)
                return
            }
            result(FlutterError.init(code: "", message: "Error in resuming chat session", details: nil))
        }
        else if(call.method == "pauseChatSession"){
            if let args = call.arguments as? NSDictionary,
               let chatRoomId = args["chatRoomId"] as? String{
                chatSessionList[chatRoomId]?.pause()
                result(nil)
                return
            }
            result(FlutterError.init(code: "", message: "Error in oausing chat session", details: nil))
        }
        else if(call.method=="closeChatSession"){
            if let args = call.arguments as? NSDictionary,
               let chatRoomId = args["chatRoomId"] as? String{
                //                chatSessionList[chatRoomId]?.
                chatSessionList.removeValue(forKey: chatRoomId)
                result(nil)
                return
            }
            result(FlutterError.init(code: "", message: "Error in closing chat session", details: nil))
            
        } else if(call.method == "getLeaderBoards"){
            if let args = call.arguments as? NSDictionary,
               let programId = args["programId"] as? String{
                self.engagementSDK.getLeaderboards(programID: programId) {res in
                    switch(res){
                    case let .success(leaderboards):
                        result(leaderboards.map { (leaderboard) -> Dictionary<String, Any> in
                            return ["id":leaderboard.id,"name":leaderboard.name,"rewardItem":[
                                "id":leaderboard.rewardItem.id,
                                "name":leaderboard.rewardItem.name
                            ]]
                        })
                    case let .failure(error):
                        result(FlutterError.init(code: "",message: error.localizedDescription,details: nil))
                    }
                }
            }
            
        } else if(call.method == "getLeaderBoardDetails"){
            if let args = call.arguments as? NSDictionary,
               let leaderBoardId = args["leaderBoardId"] as? String{
                self.engagementSDK.getLeaderboard(leaderboardID: leaderBoardId) { res in
                    switch(res){
                    case let .success(leaderboard):
                        result(["id":leaderboard.id,"name":leaderboard.name,"rewardItem":[
                            "id":leaderboard.rewardItem.id,
                            "name":leaderboard.rewardItem.name
                        ]])
                    case let .failure(error):
                        result(FlutterError.init(code: "",message: error.localizedDescription,details: nil))
                    }
                }
            }
        }
        else if(call.method == "getLeaderBoardEntryForProfile"){
            if let args = call.arguments as? NSDictionary,
               let leaderBoardId = args["leaderBoardId"] as? String,
               let profileId = args["profileId"] as? String {
                self.engagementSDK.getLeaderboardEntry(profileID: profileId, leaderboardID: leaderBoardId) { res in
                    switch(res){
                    case let .success(leaderboard):
                        result(["profile_id":leaderboard.profileId,
                                "profile_nickname":leaderboard.profileNickname,
                                "percentile_rank": leaderboard.percentileRank,
                                "rank": leaderboard.rank,
                                "score":leaderboard.score,
                                "profile":[
                                    "id":leaderboard.profile.id,
                                    "nickname":leaderboard.profile.nickname,
                                    "custom_data":leaderboard.profile.customData
                                ]
                        ])
                    case let .failure(error):
                        result(FlutterError.init(code: "",message: error.localizedDescription,details: nil))
                    }
                }
                return
            }
            result(FlutterError.init(code: "",message: "LeaderBoard Id or Profile Id cannot be null",details: nil))
        }
        else if(call.method == "getLeaderBoardEntryForCurrentUserProfile"){
            if let args = call.arguments as? NSDictionary,
               let leaderBoardId = args["leaderBoardId"] as? String{
                self.engagementSDK.getLeaderboardEntryForCurrentProfile(leaderboardID: leaderBoardId) { res in
                    switch(res){
                    case let .success(leaderboard):
                        result(["profile_id":leaderboard.profileId,
                                "profile_nickname":leaderboard.profileNickname,
                                "percentile_rank": leaderboard.percentileRank,
                                "rank": leaderboard.rank,
                                "score":leaderboard.score,
                                "profile":[
                                    "id":leaderboard.profile.id,
                                    "nickname":leaderboard.profile.nickname,
                                    "custom_data":leaderboard.profile.customData
                                ]
                        ])
                    case let .failure(error):
                        result(FlutterError.init(code: "",message: error.localizedDescription,details: nil))
                    }
                }
                return
            }
            result(FlutterError.init(code: "",message: "LeaderBoard Id cannot be null",details: nil))
        }
        else if(call.method == "getEntriesForLeaderBoard"){
            if let args = call.arguments as? NSDictionary,
               let leaderBoardId = args["leaderBoardId"] as? String,
               let pagination = args["pagination"] as? String{
                var livelikePagination = Pagination.first
                if(pagination=="first"){
                    livelikePagination = Pagination.first
                }else if(pagination=="previous"){
                    livelikePagination = Pagination.previous
                }else if(pagination=="next"){
                    livelikePagination = Pagination.next
                }
                
                self.engagementSDK.getLeaderboardEntries(leaderboardID: leaderBoardId, page: livelikePagination){ res in
                    switch(res){
                    case let .success(data):
                        result([
                            "count": data.total,
                            "hasNext" : data.hasNext,
                            "hasPrevious" : data.hasPrevious,
                            "list": data.entries.map { (entry) -> Dictionary<String, Any> in
                                return ["percentile_rank":entry.percentileRank,
                                        "profile_id":entry.profileId,
                                        "profile":[
                                            "id":entry.profile.id,
                                            "nickname":entry.profile.nickname,
                                            "custom_data":entry.profile.customData
                                        ],
                                        "profile_nickname":entry.profileNickname,
                                        "score":entry.score,
                                        "rank":entry.rank
                                ]
                            }
                        ])
                    case let .failure(error):
                        result(FlutterError.init(code: "",message: error.localizedDescription,details: nil))
                    }
                }
                return
            }
            result(FlutterError.init(code: "",message: "LeaderBoard Id cannot be null",details: nil))
        }
        else if(call.method == "fetchSponsorByProgramId"){
            if let args = call.arguments as? NSDictionary,
               let programId = args["programId"] as? String{
                let sponsor = self.engagementSDK.sponsorship
                sponsor.getBy(programID: programId) { res in
                    switch(res){
                    case let .success(sponsors):
                        result(sponsors.map { (sponsor) -> Dictionary<String, Any> in
                            return ["id":sponsor.id,"name":sponsor.name,"logo_url": sponsor.logoURL.absoluteString,"client_id":sponsor.clientID]
                        })
                    case let .failure(error):
                        result(FlutterError.init(code: "",message: error.localizedDescription,details: nil))
                    }
                }
                return
            }
            result(FlutterError.init(code: "",message: "Program Id cannot be null",details: nil))
        }
        else{
            result(FlutterMethodNotImplemented.`self`())
        }
    }
}

class FlutterUserAccessTokenStorage : AccessTokenStorage{
    private var accessToken:String?
    private let handler = EventStreamHandler()
    init(eventChannel:FlutterEventChannel,accessToken:String?=nil) {
        self.accessToken = accessToken
        eventChannel.setStreamHandler(handler)
    }
    
    func fetchAccessToken() -> String? {
        return accessToken
    }
    func storeAccessToken(accessToken: String) {
        handler.eventSink?(accessToken)
    }
}

class EventStreamHandler :NSObject, FlutterStreamHandler{
    private var _eventSink: FlutterEventSink?
    
    var eventSink: FlutterEventSink?{
        get {
            return _eventSink
        }
    }
    
    func onListen(withArguments arguments: Any?, eventSink events: @escaping FlutterEventSink) -> FlutterError? {
        _eventSink = events
        return nil
    }
    
    func onCancel(withArguments arguments: Any?) -> FlutterError? {
        _eventSink = nil
        return nil
    }
}

class ChatViewFactory: NSObject,FlutterPlatformViewFactory {
    private var messenger: FlutterBinaryMessenger
    private var plugin: SwiftLivelikeFlutterSdkPlugin
    init(messenger: FlutterBinaryMessenger,plugin:SwiftLivelikeFlutterSdkPlugin) {
        self.messenger = messenger
        self.plugin = plugin
        super.init()
    }
    
    func create(
        withFrame frame: CGRect,
        viewIdentifier viewId: Int64,
        arguments args: Any?
    ) -> FlutterPlatformView {
        return LLChatView(
            frame: frame,
            viewIdentifier: viewId,
            arguments: args,
            binaryMessenger: messenger,
            plugin:plugin
        )
    }
}

class LLChatView: NSObject, FlutterPlatformView {
    private var _view: UIView
    let chatViewController = ChatViewController()
    
    init(
        frame: CGRect,
        viewIdentifier viewId: Int64,
        arguments args: Any?,
        binaryMessenger : FlutterBinaryMessenger?,
        plugin:SwiftLivelikeFlutterSdkPlugin
    ) {
        _view = UIView()
        super.init()
        // iOS views can be created here
        createNativeView(view: _view)
        debugPrint("ChatViewID: chatView\(viewId)")
        let methodChannel = FlutterMethodChannel(name:"chatView\(viewId)",binaryMessenger: binaryMessenger!)
        methodChannel.setMethodCallHandler({
            (call: FlutterMethodCall, result: FlutterResult) -> Void in
            switch call.method{
            case "setSession":
                if let args = call.arguments as? Dictionary<String, Any>,
                   let chatRoomId = args["chatRoomId"] as? String{
                    if let session =  plugin.chatSessionList[chatRoomId]{
                        self.chatViewController.setChatSession(session)
                    } else {
                        result(FlutterError.init(code: "", message: "ChatSession not found", details: nil))
                    }
                }
                result(nil)
            default:
                result(FlutterMethodNotImplemented)
            }
        })
    }
    
    func view() -> UIView {
        return _view
    }
    
    func createNativeView(view _view: UIView){
        _view.addSubview(self.chatViewController.view)
        chatViewController.view.translatesAutoresizingMaskIntoConstraints = false
        NSLayoutConstraint.activate([
            chatViewController.view.bottomAnchor.constraint(equalTo: _view.bottomAnchor),
            chatViewController.view.topAnchor.constraint(equalTo: _view.topAnchor),
            chatViewController.view.trailingAnchor.constraint(equalTo: _view.trailingAnchor),
            chatViewController.view.leadingAnchor.constraint(equalTo: _view.leadingAnchor)
        ])
    }
}

class WidgetViewFactory: NSObject,FlutterPlatformViewFactory{
    private var messenger: FlutterBinaryMessenger
    private var plugin: SwiftLivelikeFlutterSdkPlugin
    init(messenger: FlutterBinaryMessenger,plugin:SwiftLivelikeFlutterSdkPlugin) {
        self.messenger = messenger
        self.plugin = plugin
        super.init()
    }
    
    func create(
        withFrame frame: CGRect,
        viewIdentifier viewId: Int64,
        arguments args: Any?
    ) -> FlutterPlatformView {
        return LLWidgetView(
            frame: frame,
            viewIdentifier: viewId,
            arguments: args,
            binaryMessenger: messenger,
            plugin:plugin
        )
    }
}

class LLWidgetView: NSObject, FlutterPlatformView, WidgetPopupViewControllerDelegate, LeaderboardDelegate {
    
    
    func widgetViewController(_ widgetViewController: WidgetPopupViewController, willDisplay widget: Widget) {
        print("will display => \(Int64(Date().timeIntervalSince1970 * 1000))> \(widget.kind.displayName)")
        widgetStreamHandler?.eventSink?(["show": true])
    }
    
    func widgetViewController(_ widgetViewController: WidgetPopupViewController, didDisplay widget: Widget) {
        print("did display => \(Int64(Date().timeIntervalSince1970 * 1000))> \(widget.kind.displayName)")
    }
    
    func widgetViewController(_ widgetViewController: WidgetPopupViewController, willDismiss widget: Widget) {
        print("will dismiss => \(Int64(Date().timeIntervalSince1970 * 1000))> \(widget.kind.displayName)")
    }
    
    func widgetViewController(_ widgetViewController: WidgetPopupViewController, didDismiss widget: Widget) {
        print("did dismiss => \(Int64(Date().timeIntervalSince1970 * 1000))> \(widget.kind.displayName)")
        widgetStreamHandler?.eventSink?(["show":false])
    }
    
    func widgetViewController(_ widgetViewController: WidgetPopupViewController, willEnqueueWidget widgetModel: WidgetModel) -> Widget? {
        print("will enqueu => \(widgetModel) >> \(Int64(Date().timeIntervalSince1970 * 1000))")
        let dateFormatter = DateFormatter()
        var data = [String:Any?]()
        switch widgetModel {
        case let .alert(model):
            data["program_id"] = model.programID;
            data["created_at"] = dateFormatter.string(from:model.createdAt);
            data["id"] = model.id;
            data["published_at"] = (model.publishedAt != nil) ? dateFormatter.string(from: model.publishedAt!):nil;
            data["kind"] = model.kind.displayName;
            data["custom_data"] = model.customData;
            data["text"] = model.text;
            data["image_url"] = model.imageURL?.absoluteString;
            data["link_url"] = model.linkURL?.absoluteString;
            data["link_label"] = model.linkLabel;
            data["title"] = model.title;
            data["timeout_time"] = Int(model.interactionTimeInterval)
            break;
        case let .cheerMeter(model):
            data["program_id"] = model.programID;
            data["created_at"] = dateFormatter.string(from:model.createdAt);
            data["id"] = model.id;
            data["published_at"] = (model.publishedAt != nil) ? dateFormatter.string(from: model.publishedAt!):nil;
            data["kind"] = model.kind.displayName;
            data["custom_data"] = model.customData;
            data["title"] = model.title
            data["timeout_time"] = Int(model.interactionTimeInterval)
            
            var dict: [Dictionary<String, Any?>] = []
            for option in model.options {
                dict.append( ["id" : option.id,
                              "image_url":option.imageURL.absoluteString,
                              "text":option.text,
                              "vote_count": option.voteCount
                ])
            }
            data["options"] = dict
            break;
        case let .quiz(model):
            data["program_id"] = model.programID;
            data["created_at"] = dateFormatter.string(from:model.createdAt);
            data["id"] = model.id;
            data["published_at"] = (model.publishedAt != nil) ? dateFormatter.string(from: model.publishedAt!):nil;
            data["kind"] = model.kind.displayName;
            data["custom_data"] = model.customData;
            data["question"] = model.question;
            data["timeout_time"] = Int(model.interactionTimeInterval)
            
            var dict: [Dictionary<String, Any?>] = []
            for option in model.choices {
                dict.append(["id" : option.id,
                             "image_url":option.imageURL?.absoluteString,
                             "text":option.text,
                             "is_correct":option.isCorrect,
                             "answer_count": option.answerCount
                ])
            }
            data["choices"] = dict
            break;
        case let .prediction(model):
            data["program_id"] = model.programID;
            data["created_at"] = dateFormatter.string(from:model.createdAt);
            data["id"] = model.id;
            data["published_at"] = (model.publishedAt != nil) ? dateFormatter.string(from: model.publishedAt!):nil;
            data["kind"] = model.kind.displayName;
            data["custom_data"] = model.customData;
            data["question"] = model.question;
            data["timeout_time"] = Int(model.interactionTimeInterval)
            
            var dict: [Dictionary<String, Any?>] = []
            for option in model.options {
                dict.append( ["id" : option.id,
                              "image_url":option.imageURL?.absoluteString,
                              "text":option.text,
                              "vote_count": option.voteCount
                ])
            }
            data["options"] = dict
            break;
        case let .predictionFollowUp(model):
            data["program_id"] = model.programID;
            data["created_at"] = dateFormatter.string(from:model.createdAt);
            data["id"] = model.id;
            data["published_at"] = (model.publishedAt != nil) ? dateFormatter.string(from: model.publishedAt!):nil;
            data["kind"] = model.kind.displayName;
            data["custom_data"] = model.customData;
            data["question"] = model.question;
            data["timeout_time"] = Int(model.interactionTimeInterval)
            
            var dict: [Dictionary<String, Any?>] = []
            for option in model.options {
                dict.append(["id" : option.id,
                             "image_url":option.imageURL?.absoluteString,
                             "text":option.text,
                             "vote_count": option.voteCount
                ])
            }
            data["options"] = dict
            break;
        case  let .poll(model):
            data["program_id"] = model.programID;
            data["created_at"] = dateFormatter.string(from:model.createdAt);
            data["id"] = model.id;
            data["published_at"] = (model.publishedAt != nil) ? dateFormatter.string(from: model.publishedAt!):nil;
            data["kind"] = model.kind.displayName;
            data["custom_data"] = model.customData;
            data["question"] = model.question;
            data["timeout_time"] = Int(model.interactionTimeInterval)
            
            var dict: [Dictionary<String, Any?>] = []
            for option in model.options {
                dict.append(["id" : option.id,
                             "image_url":option.imageURL?.absoluteString,
                             "text":option.text,
                             "vote_count": option.voteCount
                ])
            }
            data["options"] = dict
            break;
        case let .imageSlider(model):
            data["program_id"] = model.programID;
            data["created_at"] = dateFormatter.string(from:model.createdAt);
            data["id"] = model.id;
            data["published_at"] = (model.publishedAt != nil) ? dateFormatter.string(from: model.publishedAt!):nil;
            data["kind"] = model.kind.displayName;
            data["custom_data"] = model.customData;
            data["question"] = model.question;
            var dict: [Dictionary<String, Any?>] = []
            for option in model.options {
                dict.append(["id" : option.id,
                             "image_url":option.imageURL.absoluteString,
                ])
            }
            data["options"] = dict
            data["timeout_time"] = Int(model.interactionTimeInterval)
            
            data["initial_magnitude"] = model.initialMagnitude;
            data["average_magnitude"] = model.averageMagnitude;
            break;
        case let .socialEmbed(model):
            data["created_at"] = dateFormatter.string(from:model.createdAt);
            data["id"] = model.id;
            data["published_at"] = (model.publishedAt != nil) ? dateFormatter.string(from: model.publishedAt!):nil;
            data["kind"] = model.kind.displayName;
            data["custom_data"] = model.customData;
            data["comment"] = model.comment;
            data["timeout_time"] = Int(model.interactionTimeInterval)
            break;
        }
        let widget = DefaultWidgetFactory.makeWidget(from: widgetModel)
        return widget
    }
    
    private var _view: UIView
    let widgetViewController = WidgetPopupViewController()
    var widgetStreamHandler: EventStreamHandler?
    var rewardStreamHandler: EventStreamHandler?
    
    func leaderboard(_ leaderboardClient: LeaderboardClient, currentPositionDidChange position: LeaderboardPosition) {
        //        debugPrint("Rewards: \(position.percentileRank), \(position.rank) , \(position.score) ")
    }
    
    init(
        frame: CGRect,
        viewIdentifier viewId: Int64,
        arguments args: Any?,
        binaryMessenger : FlutterBinaryMessenger?,
        plugin:SwiftLivelikeFlutterSdkPlugin
    ) {
        _view = UIView()
        super.init()
        // iOS views can be created here
        createNativeView(view: _view)
        debugPrint("WidgetView ID : widgetView\(viewId)")
        self.rewardStreamHandler = plugin.rewardStreamHandler
        let methodChannel = FlutterMethodChannel(name:"widgetView\(viewId)",binaryMessenger: binaryMessenger!)
        methodChannel.setMethodCallHandler({
            (call: FlutterMethodCall, result: FlutterResult) -> Void in
            switch call.method {
            case "setSession":
                if let args = call.arguments as? Dictionary<String, Any>,
                   let programId = args["programId"] as? String {
                    let session =  plugin.sessionList[programId]
                    self.widgetViewController.session  = session
                    self.widgetViewController.delegate = self
                    self.widgetStreamHandler = plugin.widgetStreamHandler
                    //                    session?.getLeaderboardClients { result in
                    //                        switch result {
                    //                        case let .success(leaderboardClients):
                    //                            guard let leaderboard = leaderboardClients.first else { return }
                    //                            leaderboard.delegate = self
                    //                        case let .failure(error):
                    //                            print(error)
                    //                        }
                    //                    }
                }
                result(nil)
            default:
                result(FlutterMethodNotImplemented)
            }
        })
    }
    
    func view() -> UIView {
        return _view
    }
    
    func createNativeView(view _view: UIView){
        _view.addSubview(self.widgetViewController.view)
        // Applying constraints to fill the container
        widgetViewController.view.translatesAutoresizingMaskIntoConstraints = false
        NSLayoutConstraint.activate([
            widgetViewController.view.bottomAnchor.constraint(equalTo: _view.bottomAnchor),
            widgetViewController.view.topAnchor.constraint(equalTo: _view.topAnchor),
            widgetViewController.view.trailingAnchor.constraint(equalTo: _view.trailingAnchor),
            widgetViewController.view.leadingAnchor.constraint(equalTo: _view.leadingAnchor)
        ])
    }
}
